==> alh_spider-2.0.0.tgz: package/spider.js <==


var http = require("http");
var nconf = require('nconf');
nconf.file('./config.json');
var redis = require('redis');

var async = require("async");
client = redis.createClient(nconf.get('port'),nconf.get('redisip'),{});

//////////////////////////////////////////////////////

// var obj={};
// obj.url="http://yuedu.163.com/#f=topnav";
// obj.parenturl="http://yuedu.163.com/#f=topnav";
// var inistr=JSON.stringify(obj);
// client.lpush("url_seed",inistr,redis.print);

//////////////////////////////////////////////////////

exports.main = main;

//process.exit();
//hset url_seed_list http://www.expressjs.com.cn ""

function writedb(url,parenturl,st,callback)
{
	//console.log("writedb");
	if (st=="pass") {
		client.hset("url_list",url,parenturl,function(err,rt){
			if (err) {
				console.log("hset:"+err);
			}
			console.log("Pass URL:"+url);
			callback();
		});
	}
	if (st=="seed") {
		client.hexists("url_list",url,function(err1,rt1){
			if (rt1==0) {
				client.hexists("url_error",url,function(err2,rt2){
					if (rt2==0) {
						var obj={};
						obj.url=url;
						obj.parenturl=parenturl;
						var jsondata=JSON.stringify(obj);
						client.lpush("url_seed",jsondata,function(){
							callback();
						});
					}
					else{
						callback();
					}
				});
			}
			else{
				callback();
			}
		});
	}
	if (st=="error") {
		client.hset("url_error",url,parenturl,function(){
			callback();
		});
	}
}

function gethtml(url,parenturl,callback)
{
	try{
		var jsdom = require("jsdom");
	    jsdom.env({url: url,
	    	src: [jquery],
	    	done: function (errors, window) {
				if (errors) {
					console.log("jsdom error:"+errors);
					if (window!=null && window!=undefined) {
						window.close();
					}
					callback("error");
				}
				else{
					if (window==undefined || window==null) {
						callback("error");
					}
					else{
						var i=0;
						console.log("HREF COUNT:"+window.$("a").length);
						async.whilst(
							function(){
								return i<window.$("a").length;
							},
							function(next){
								var href=window.$("a:eq("+i+")").attr("href");
								checkurl(href,url,function(isurl,geturl){
									if (isurl==true) {
										writedb(geturl,url,"seed",function(){
										 	i++;
										 	next();
										});
									}
									else{
										console.log("无效网址:"+href);
										i++;
									 	next();
									}
								});

							},
							function(err,n){
								if (window!=null && window!=undefined) {
									window.close();
								}
								callback("pass");
							}
						);
					}
				}
			}
		});
	}
	catch(e)
	{
		console.log('jsdom.env error:'+e);
		callback("error");
	}
}

function checkurl(url,parenturl,callback)
{
	if (url!=null && url!="") {
		var tempurl=url.toLowerCase();
		var regx1=/http:\/\//;
		var regx2=/https:\/\//;
		var rt1=regx1.test(tempurl);
		var rt2=regx2.test(tempurl);
		if (rt1 || rt2) {
			callback(true,url);
		}
		else if (url.trim()=="#" || url.trim()=="/" || url.trim()=="/#") {
			//console.log(url);
			callback(false,"");
		}
		else if (rt1==false && rt2==false) {
			if (parenturl==""||parenturl==null) {
				callback(true,'http://'+url);
			}
			var purl=parenturl.replace(/\\/g,'/');
			var urlhead=purl.match(/(http|https):\/\/[^\/]+/);
			if (url[0]=='/' || url[0]=='\\') {
				callback(true,urlhead[0]+url);
			}
			else{
				callback(true,urlhead[0]+'/'+url);
			}
		}
	}
	else
	{
		callback(false,"");
	}
}


function main()
{
	var fs = require("fs");
	var jquery = fs.readFileSync("./jquery.js", "utf-8");
	async.whilst(
		function(){
			return true;
		},
		function(callback){
			client.rpop("url_seed",function(err,rt){
				if (rt!=null) {
					var obj=JSON.parse(rt);
					var url=obj.url;
					var parenturl=obj.parenturl;

					console.log("GETHTML:"+url);

					client.hexists("url_list",url,function(err1,rt1){
						if (rt1==0) {
							client.hexists("url_error",url,function(err2,rt2){
								if (rt2==0) {
									gethtml(url,parenturl,function(result){
										writedb(url,parenturl,result,function(){
											callback();
										});
									});
								}
								else{
									callback();
								}
							});
						}
						else{
							callback();
						}
					});
				}
				else{
					callback();
				}
			});
		},
		function(err,n){
			process.exit();
		}
	);
}

==> alh_spider-15.0.0.tgz: package/spider.js <==


//var http = require("http");
//var nconf = require('nconf');
//nconf.file('./node_modules/alh_spider/config.json');
var redis = require('redis');

var async = require("async");
var client;
var dbhead="";

//////////////////////////////////////////////////////
var fs = require("fs");
var jquery = fs.readFileSync("./node_modules/alh_spider/jquery.js", "utf-8");


//////////////////////////////////////////////////////

exports.run = run;
//exports.main = main;

//process.exit();
//hset url_seed_list http://www.expressjs.com.cn ""

function writedb(url,parenturl,st,callback)
{
	//console.log("writedb");
	if (st=="pass") {
		client.hset(dbhead+"url_list",url,parenturl,function(err,rt){
			if (err) {
				console.log("hset:"+err);
			}
			console.log("Pass URL:"+url);
			callback();
		});
	}
	if (st=="seed") {
		client.hexists(dbhead+"url_list",url,function(err1,rt1){
			if (rt1==0) {
				client.hexists(dbhead+"url_error",url,function(err2,rt2){
					if (rt2==0) {
						var obj={};
						obj.url=url;
						obj.parenturl=parenturl;
						var jsondata=JSON.stringify(obj);
						client.lpush(dbhead+"url_seed",jsondata,function(){
							callback();
						});
					}
					else{
						callback();
					}
				});
			}
			else{
				callback();
			}
		});
	}
	if (st=="error") {
		client.hset(dbhead+"url_error",url,parenturl,function(){
			callback();
		});
	}
}

function gethtml(url,parenturl,callback)
{
	//try{
		var jsdom = require("jsdom");
	    jsdom.env({url: url,
	    	src: [jquery],
	    	done: function (errors, window) {
				if (errors) {
					console.log("jsdom error:"+errors);
					if (window!=null && window!=undefined) {
						window.close();
					}
					callback("error");
				}
				else{
					if (window==undefined || window==null) {
						callback("error");
					}
					else{
						var i=0;
						console.log("HREF COUNT:"+window.$("a").length);
						async.whilst(
							function(){
								return i<window.$("a").length;
							},
							function(next){
								var href=window.$("a:eq("+i+")").attr("href");
								checkurl(href,url,function(isurl,geturl){
									if (isurl==true) {
										writedb(geturl,url,"seed",function(){
										 	i++;
										 	next();
										});
									}
									else{
										//console.log("无效网址:"+href);
										i++;
									 	next();
									}
								});

							},
							function(err,n){
								if (window!=null && window!=undefined) {
									window.close();
								}
								callback("pass");
							}
						);
					}
				}
			}
		});
	//}
	// catch(e)
	// {
	// 	console.log('jsdom.env error:'+e);
	// 	callback("error");
	// }
	process.on('uncaughtException', function (err) {
    	console.log(err);
    	callback("error");
	});
}

function checkurl(url,parenturl,callback)
{
	if (url!=null && url!="") {
		var tempurl=url.toLowerCase();
		var regx1=/http:\/\//;
		var regx2=/https:\/\//;
		var rt1=regx1.test(tempurl);
		var rt2=regx2.test(tempurl);
		if (rt1 || rt2) {
			callback(true,url);
		}
		else if (url.trim()=="#" || url.trim()=="/" || url.trim()=="/#") {
			//console.log(url);
			callback(false,"");
		}
		else if(tempurl.indexOf("javascript:")!=-1){
			callback(false,"");
		}
		else if (rt1==false && rt2==false) {
			if (parenturl==""||parenturl==null) {
				callback(true,'http://'+url);
			}
			var purl=parenturl.replace(/\\/g,'/');
			var urlhead=purl.match(/(http|https):\/\/[^\/]+/);
			if (url[0]=='/' || url[0]=='\\') {
				callback(true,urlhead[0]+url);
			}
			else{
				callback(true,urlhead[0]+'/'+url);
			}
		}
	}
	else
	{
		callback(false,"");
	}
}


function main()
{
	async.whilst(
		function(){
			return true;
		},
		function(callback){
			client.rpop(dbhead+"url_seed",function(err,rt){
				if (rt!=null) {
					var obj=JSON.parse(rt);
					var url=obj.url;
					var parenturl=obj.parenturl;

					console.log("GETHTML:"+url);

					client.hexists(dbhead+"url_list",url,function(err1,rt1){
						if (rt1==0) {
							client.hexists(dbhead+"url_error",url,function(err2,rt2){
								if (rt2==0) {
									gethtml(url,parenturl,function(result){
										writedb(url,parenturl,result,function(){
											callback();
										});
									});
								}
								else{
									callback();
								}
							});
						}
						else{
							callback();
						}
					});
				}
				else{
					callback("null",0);
				}
			});
		},
		function(err,n){
			process.exit();
		}
	);
}

function run(redis_ip,port,dbmark,dbn,url)
{
	dbhead = dbmark;
	client = redis.createClient(port,redis_ip,{});
	var obj={};
	client.select(dbn,function(){
		if (url!=null && url!=undefined) {
			obj.url=url;
			obj.parenturl=url;
			var inistr=JSON.stringify(obj);
			client.rpush(dbhead+"url_seed",inistr,function(){
				main();
			});
		}
		else{
			main();
		}
	});
}
